#include "ppc.h"
#include "m3x3.h"

#include <math.h>
#include <float.h>

PPC::PPC(float hfov, int _w, int _h) : 
  w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  float f = (float)w/(2.0f*tanf(hfov/2.0f/180.0f*3.1415f));
  c = V3(-(float)w/2.0f, (float)h/2.0f, -f);

}

bool PPC::Project(V3 P, V3 &projP) {

  projP = V3(FLT_MAX, FLT_MAX, FLT_MAX);
  M3x3 cam;
  cam.SetColumn(0, a);
  cam.SetColumn(1, b);
  cam.SetColumn(2, c);
  M3x3 invcam = cam.Invert();
  V3 q = invcam*(P-C);
  if (q[2] < 0.0f)
    return false;
  projP[0] = q[0]/q[2];
  projP[1] = q[1]/q[2];
  projP[2] = 1.0f/q[2];
  return true;

}

void PPC::TranslateFB(float step) {

  V3 vd = (a^b);
  vd = vd / vd.Length();
  C = C + vd*step;

}


void PPC::Pan(float rstep) {

  V3 axis = b.Normalized()*-1.0f;
  a = a.RotateThisVectorAboutAxis(axis, rstep);
  c = c.RotateThisVectorAboutAxis(axis, rstep);

}

void PPC::TranslateLR(float step) {

  V3 vd = a.Normalized();
  C = C + vd*step;

}

void PPC::TranslateUD(float step) {

  V3 vd = b.Normalized()*-1.0f;
  C = C + vd*step;

}


void PPC::Tilt(float rstep) {

  V3 axis = a.Normalized();
  b = b.RotateThisVectorAboutAxis(axis, rstep);
  c = c.RotateThisVectorAboutAxis(axis, rstep);

}
